@@ -25,6 +25,7 @@
 #  PYTHONPATH="$PYTHONPATH:$samba4srcdir/torture/drs/python" $SUBUNITRUN fsmo -U"$DOMAIN/$DC_USERNAME"%"$DC_PASSWORD"
 #
 
+from __future__ import print_function
 import sys
 import time
 import os
@@ -50,13 +51,15 @@ class DrsFsmoTestCase(drs_base.DrsBaseTestCase):
         self.infrastructure_dn = "CN=Infrastructure," + self.domain_dn
         self.naming_dn = "CN=Partitions," + self.config_dn
         self.rid_dn = "CN=RID Manager$,CN=System," + self.domain_dn
+        self.domain_dns_dn = (
+            "CN=Infrastructure,DC=DomainDnsZones, %s" % self.domain_dn )
+        self.forest_dns_dn = (
+            "CN=Infrastructure,DC=ForestDnsZones, %s" % self.domain_dn )
 
     def tearDown(self):
         super(DrsFsmoTestCase, self).tearDown()
 
     def _net_fsmo_role_transfer(self, DC, role, noop=False):
-        # find out where is samba-tool command
-        net_cmd = os.path.abspath("./bin/samba-tool")
         # make command line credentials string
         ccache_name = self.get_creds_ccache_name()
         cmd_line_auth = "--krb5-ccache=%s" % ccache_name
@@ -98,7 +101,7 @@ class DrsFsmoTestCase(drs_base.DrsBaseTestCase):
         """Triggers transfer of role from DC1 to DC2
            and vice versa so the role goes back to the original dc"""
         # dc2 gets the role from dc1
-        print "Testing for %s role transfer from %s to %s" % (role, self.dnsname_dc1, self.dnsname_dc2)
+        print("Testing for %s role transfer from %s to %s" % (role, self.dnsname_dc1, self.dnsname_dc2))
 
         self._net_fsmo_role_transfer(DC=self.dnsname_dc2, role=role)
         # check if the role is transfered
@@ -109,7 +112,7 @@ class DrsFsmoTestCase(drs_base.DrsBaseTestCase):
                         "Transferring %s role to %s has failed, master is: %s!"%(role, self.dsServiceName_dc2, master))
 
         # dc1 gets back the role from dc2
-        print "Testing for %s role transfer from %s to %s" % (role, self.dnsname_dc2, self.dnsname_dc1)
+        print("Testing for %s role transfer from %s to %s" % (role, self.dnsname_dc2, self.dnsname_dc1))
         self._net_fsmo_role_transfer(DC=self.dnsname_dc1, role=role)
         # check if the role is transfered
         (res, master) = self._wait_for_role_transfer(ldb_dc=self.ldb_dc1,
@@ -119,7 +122,7 @@ class DrsFsmoTestCase(drs_base.DrsBaseTestCase):
                         "Transferring %s role to %s has failed, master is: %s!"%(role, self.dsServiceName_dc1, master))
 
         # dc1 keeps the role
-        print "Testing for no-op %s role transfer from %s to %s" % (role, self.dnsname_dc2, self.dnsname_dc1)
+        print("Testing for no-op %s role transfer from %s to %s" % (role, self.dnsname_dc2, self.dnsname_dc1))
         self._net_fsmo_role_transfer(DC=self.dnsname_dc1, role=role, noop=True)
         # check if the role is transfered
         (res, master) = self._wait_for_role_transfer(ldb_dc=self.ldb_dc1,
@@ -142,3 +145,9 @@ class DrsFsmoTestCase(drs_base.DrsBaseTestCase):
 
     def test_NamingMasterTransfer(self):
         self._role_transfer(role="naming", role_dn=self.naming_dn)
+
+    def test_DomainDnsZonesMasterTransfer(self):
+        self._role_transfer(role="domaindns", role_dn=self.domain_dns_dn)
+
+    def test_ForestDnsZonesMasterTransfer(self):
+        self._role_transfer(role="forestdns", role_dn=self.forest_dns_dn)
